-- import: IDE+COL, IPL (xref dependencies)

fn rootOf obj = (
	while obj.parent != undefined do obj = obj.parent
	obj
)

fn headOf obj = (
	root = rootOf obj
	if superClassOf root == GeometryClass do
		return root
	for c in root.children do
		if matchPattern c.name pattern:"*_l0" do
			return c
	undefined
)

fn deleteDummies root = (
	for c in root.children do (
		deleteDummies c
		if classOf c == Dummy do
			delete c
	)
)

fn importMapDff name = (
	dffImp.prepend = 1
	dffImp.autosmooth = 1
	dffImp.smoothingAngle = 45
	dffImp.convertHierarchy = 1
	importFile name #noPrompt using:RenderWare_model
	for s in selection do
		s.name = substring s.name 2 -1
	x = rootOf $selection[1]
	if classOf x == Dummy do (
		myname = x.name
		-- TODO: save xform?
		delete x
		g = group $selection name:myname
-- 		setGroupOpen g true
		x = rootOf $selection[1]
	)
	deleteDummies x
	return x
)

fn placeObject obj = (
	global lastobj
	obj = rootOf obj
	if lastobj == undefined do (
		lastobj = obj
		return 0
	)
	obj.pos.y = -obj.min.y + lastobj.max.y + 10
	lastobj = obj
)

dffdir = "D:/rockstargames/ps2/gta3/MODELS/gta3_archive/"


fn objsLine line = (
	dist = #()
	id = readValue line
	model = trimleft (readDelimitedString line ",")
	texdict = trimleft (readDelimitedString line ",")
	nobj = readValue line
	for i = 1 to nobj do
		dist[i] = readValue line
	flags = readValue line

	dffpath = dffdir + model + ".dff"
	root = importMapDff dffpath
	obj = headOf root
	custAttributes.add obj gtaAttrib

	c = findCol coldict model
	if c != undefined do  (
		objs = createColF c.file c.val
		layer = LayerManager.getLayerFromName "Collision"
		if layer == undefined do
			layer = LayerManager.newLayerFromName "Collision"
		for o in objs do (
			layer.addNode o
			attachObjects obj o move:false
		)
	)

	obj.id = id
	obj.txd = texdict
	obj.dist1 = dist[1]
	if nobj > 1 do obj.dist2 = dist[2]
	if nobj > 2 do obj.dist3 = dist[3]
	setFlags obj flags
	modelInfo[id] = obj
	placeObject obj
	obj
)

fn tobjLine line = (
	obj = objsLine line
	setTimed obj true
	obj.timeon = readvalue line
	obj.timeoff = readvalue line
)

fn getById id = (
	for obj in objects where ((isMapObj obj) and (obj.id == id)) collect obj
)



fn makeseg spl A B = (
	seg = addNewSpline spl
	addKnot spl seg #corner #line A
	addKnot spl seg #corner #line B
	updateShape spl
)

struct pathknot (
	nodetype,
	next,
	iscross,
	pos,
	width,
	left,
	right
)

fn pathLine line file = (
	type = trimleft (readDelimitedString line ",")
	id = readValue line
	name = trimleft (readDelimitedString line ",")
	obj = modelInfo[id]

	knots = #()
	for i = 1 to 12 do (
		line = (readline file) as stringstream
		nodetype = readValue line
		if nodetype == 0 do
			continue
		next = (readValue line) + 1
		iscross = readValue line
		x = (readValue line)/16.0
		y = (readValue line)/16.0
		z = (readValue line)/16.0
		pos = [x, y, z]
		width = (readValue line)/16.0
		left = readValue line
		right = readValue line
		append knots (pathKnot nodetype:nodetype next:next iscross:iscross pos:pos width:width left:left right:right)
	)
	pos = [0, 0, 0]
	for k in knots do (
		if k.nodetype == 1 do
			pos = k.pos
	)
	spl = SplineShape pos:pos name:(uniquename "PathSegment")
	for i = 1 to knots.count do (
		if knots[i].next > 0 do
			makeseg spl knots[i].pos knots[knots[i].next].pos
	)
	spl.wirecolor = case type of (
	"ped": (color 255 0 0)
	"car": (color 0 255 0)
	)
	if obj != undefined do (
		spl.pos += obj.pos	-- object is repositioned according to bbox
		attachObjects obj spl move:false
	)
)

fn twodfxLine line = (
	ln = line as string
	id = readValue line
	x = readValue line
	y = readValue line
	z = readValue line
	r = readValue line
	g = readValue line
	b = readValue line
	unk = readValue line
	type = readValue line
	case type of (
	0:	(-- light	Omni
		corona = trimleft (readDelimitedString line ",")
		shadow = trimleft (readDelimitedString line ",")
		dist = readValue line
		outRange = readValue line
		size = readValue line
		innerRange = readValue line
		shdIntens = readValue line
		flash = readValue line
		wet = readValue line
		flare = readValue line
		flags = readValue line
		fx = omnilight pos:[x, y, z]
		fx.rgb = (color r g b)
	)
	1:	(-- particle	Spray
		particletype = readValue line	-- 0, 1, 2 used by IDEs
		dirx = readValue line
		diry = readValue line
		dirz = readValue line
		scale = readValue line
		fx = spray pos:[x, y, z]
	)
	2:	(-- some kind of ped attractor	CamPoint
		flag = readValue line		-- always 1; 0 unused
		dirx = readValue line
		diry = readValue line
		dirz = readValue line
		probability = readValue line
		fx = campoint pos:[x, y, z]
-- 		print ([dirx, diry, dirz] as string)
-- 		print (dirx*dirx + diry*diry)
	)
	)

	obj = modelInfo[id]
	if obj != undefined do (
		fx.pos += obj.pos	-- object is repositioned according to bbox
		attachObjects obj fx move:false
	)
)

fn readIDE filename = (
	sections = #("objs", "tobj", "path", "2dfx", "end")
	inf = openFile filename
	if inf == undefined do
		return 0
	sect = #end
	global lastobj = undefined
	while not eof inf do (
		line = readline inf
		if line == "" or line[1] == "#" do
			continue
		i = findItem sections line
		if i > 0 do (
			sect = sections[i] as name
-- 			print sect
			continue
		)
		case sect of (
		#objs: objsLine (line as stringstream)
		#tobj: tobjLine (line as stringstream)
		#path: pathLine (line as stringstream) inf
		#2dfx: twodfxLine (line as stringstream)
		)
	)
	close inf
)

fn importXref name = (
	for i = 1 to xrefFiles.count do (
		contents = xrefContents[i]
		if (findItem contents name) != 0 do
			return xrefs.addNewXRefObject xrefFiles[i] name dupMtlNameAction:#autoRename
		if (findItem contents (name + "_l0")) != 0 do
			return xrefs.addNewXRefObject xrefFiles[i] (name + "_l0") dupMtlNameAction:#autoRename
		if (findItem contents (name + "_L0")) != 0 do
			return xrefs.addNewXRefObject xrefFiles[i] (name + "_L0") dupMtlNameAction:#autoRename
	)
	undefined
)

fn instLine line = (
	ln = line as string
	id = readValue line
	model = trimleft (readDelimitedString line ",")
	x = readValue line
	y = readValue line
	z = readValue line
	pos =  [x, y, z]
	x = readValue line
	y = readValue line
	z = readValue line
	scl = [x, y, z]
	x = readValue line
	y = readValue line
	z = readValue line
	w = readValue line
	rot = (quat x y z -w)
	-- this is rather ugly but what the heck
	global modelInfo
	obj = modelInfo[id]
	if obj != undefined then (
		if (insts[id] == undefined) then (
			-- original object imported from IDE
			inst = rootOf obj
			inst.rotation = rot
			inst.scale = scl
			inst.pos = pos
			insts[id] = 1
		) else (
			-- instance of IDE defined object
			inst = instance obj
			inst.name = obj.name
			inst.rotation = rot
			inst.scale = scl
			inst.pos = pos
		)
	) else (
		if (insts[id] == undefined) then (
			-- import XRef
			inst = importXref model
			if inst == undefined do (
				format "warning: model % % not found\n" id model
				return undefined
			)
			inst.rotation = rot
			inst.scale = scl
			inst.pos = pos
			insts[id] = inst
		) else (
			-- instance of an XRef
			inst = instance insts[id]
			inst.name = insts[id].name
			inst.rotation = rot
			inst.scale = scl
			inst.pos = pos
		)
	)
)

fn readIPL filename = (
	sections = #("inst", "cull", "pick", "end")
	inf = openFile filename
	if inf == undefined do
		return 0
	sect = #end
	global insts = #()
	while not eof inf do (
		line = readline inf
		if line == "" or line[1] == "#" do
			continue
		i = findItem sections line
		if i > 0 do (
			sect = sections[i] as name
-- 			print sect
			continue
		)
		case sect of (
		#inst: instLine (line as stringstream)
		)
	)
	close inf
)

xrefFiles = #("roadpart.max",
	"miscobj.max",
	"barriers.max",
	"dynamic.max",
	"vegepart.max",
	"temppart.max",
	"making.max",
	"indroads.max")

xrefContents = for file in xrefFiles collect getMAXFileObjectNames (xrefPaths.getFullFilePath file)

mapsdir = "C:/Users/aap/Desktop/3map/"

fn mapfile name ext = (
	mapsdir + name + "/" + name + "." + ext
)

coldict = #()
modelInfo = #()
addToColIndexF coldict "C:/Users/aap/Desktop/3map/generic.col"
addToColIndexF coldict "C:/Users/aap/Desktop/3map/industsw/industsw.col"
addToColIndexF coldict "C:/Users/aap/Desktop/3map/industse/industse.col"
addToColIndexF coldict "C:/Users/aap/Desktop/3map/indroads/indroads.col"
addToColIndexF coldict "C:/Users/aap/Desktop/3map/temppart/temppart.col"
addToColIndexF coldict "C:/Users/aap/Desktop/3map/making/making.col"

-- readIDE "C:/Users/aap/Desktop/3map/generic/roadpart.ide"
-- readIDE "C:/Users/aap/Desktop/3map/generic/props1.ide"
-- readIDE "C:/Users/aap/Desktop/3map/generic/barriers.ide"
-- readIDE "C:/Users/aap/Desktop/3map/generic/dynamic.ide"
-- readIDE "C:/Users/aap/Desktop/3map/generic/vegepart.ide"

-- c = findCol coldict "chinaquadhouse"
-- objs = createColF c.file c.val

importMapDff (dffdir + "/lamppost1.dff")

-- readIDE "C:/Users/aap/Desktop/3map/indroads/indroads.ide"
-- readIPL "C:/Users/aap/Desktop/3map/indroads/indroads.ipl"
-- readIDE "C:/Users/aap/Desktop/3map/temppart/temppart.ide"
-- readIPL "C:/Users/aap/Desktop/3map/temppart/temppart.ipl"
-- readIDE "C:/Users/aap/Desktop/3map/making/making.ide"
-- readIPL "C:/Users/aap/Desktop/3map/making/making.ipl"
-- readIDE "C:/Users/aap/Desktop/3map/industsw/industsw.ide"
-- readIPL "C:/Users/aap/Desktop/3map/industsw/industsw.ipl"
-- readIDE "C:/Users/aap/Desktop/3map/industse/industse.ide"
-- readIPL "C:/Users/aap/Desktop/3map/industse/industse.ipl"
-- readIDE "C:/Users/aap/Desktop/3map/test.ide"
-- readIPL "C:/Users/aap/Desktop/3map/test.ipl"
modelInfo = #()
insts = #()
--readIDE "D:/rockstargames/ps2/gta3/DATA/MAPS/gta3.ide"

--lodname = "LOD"+ (substring $.name 4 -1)
--select (getnodebyname lodname all:true)